import pytest

from tests.defaults.conftest import ALEMBIC_CONFIG
from tests.utils import run_alembic_command

from .flask_app import db


@pytest.mark.usefixtures("test_client")
class TestAuditLoggerAlembicHooks:
    def test_initialize_file(self):
        with open(ALEMBIC_CONFIG / "versions" / "1_create.py") as f:
            migration_contents = f.read()

        parts = migration_contents.split("\n\n")
        upgrade = parts[-2]
        assert "def upgrade():" in upgrade
        assert "op.init_audit_logger_extension('btree_gist')" in upgrade
        assert (
            "op.init_audit_logger_schema()" not in upgrade
        )  # only present when separate schema configured
        assert "op.create_table('transaction'" in upgrade
        assert "op.create_table('activity'" in upgrade
        assert (
            "op.create_index(op.f('ix_activity_native_transaction_id'), 'activity', ['native_transaction_id'], unique=False)"
            in upgrade
        )
        assert (
            "op.init_audit_logger_function('get_setting(setting text, fallback text)')" in upgrade
        )
        assert "op.init_audit_logger_function('jsonb_subtract(arg1 jsonb, arg2 jsonb)')" in upgrade
        assert "op.init_audit_logger_function('create_activity()')" in upgrade
        assert "op.init_audit_logger_triggers('article')" in upgrade
        assert "op.init_audit_logger_triggers('user')" in upgrade

        downgrade = parts[-1]
        assert "op.remove_audit_logger_triggers('user')" in downgrade
        assert "op.remove_audit_logger_triggers('article')" in downgrade
        assert "op.remove_audit_logger_function('create_activity()')" in downgrade
        assert (
            "op.remove_audit_logger_function('jsonb_subtract(arg1 jsonb, arg2 jsonb)')"
            in downgrade
        )
        assert (
            "op.remove_audit_logger_function('get_setting(setting text, fallback text)')"
            in downgrade
        )
        assert (
            "op.drop_index(op.f('ix_activity_native_transaction_id'), table_name='activity')"
            in downgrade
        )
        assert "op.drop_table('activity')" in downgrade
        assert "op.drop_table('transaction')" in downgrade
        assert "op.drop_table('user')" in downgrade
        assert "op.drop_table('article')" in downgrade
        assert "op.remove_audit_logger_extension('btree_gist')" in downgrade

    def test_second_file_is_empty(self):
        output = run_alembic_command(
            engine=db.engine,
            command="revision",
            command_kwargs={"autogenerate": True, "rev_id": "2", "message": "test"},
            alembic_config=ALEMBIC_CONFIG,
        )

        assert "versions/2_test.py" not in output
        assert "No changes in schema detected" in output



import pytest
from sqlalchemy.orm import scoped_session, sessionmaker

from tests.custom_actor.flask_app import SuperUser, app, audit_logger, db
from tests.utils import REPO_ROOT, run_audit_logger_migrations

ALEMBIC_CONFIG = REPO_ROOT / "tests" / "custom_actor" / "alembic_config"


# @pytest.fixture
@pytest.fixture(scope="session")
def test_client():
    test_client = app.test_client()

    with app.app_context():
        with run_audit_logger_migrations(db, ALEMBIC_CONFIG):
            yield test_client


@pytest.fixture(autouse=True)
def enable_transactional_tests(test_client):
    """https://docs.sqlalchemy.org/en/20/orm/session_transaction.html#joining-a-session-into-an-external-transaction-such-as-for-test-suites"""
    connection = db.engine.connect()
    transaction = connection.begin()

    db.session = scoped_session(
        session_factory=sessionmaker(
            bind=connection,
            join_transaction_mode="create_savepoint",
        )
    )

    yield

    db.session.close()
    transaction.rollback()
    connection.close()


@pytest.fixture
def superuser(test_client):
    su = SuperUser(id=123)
    db.session.add(su)
    with audit_logger.disable(db.session):
        db.session.commit()

    with test_client.application.test_request_context():
        yield su


# @pytest.fixture
# def user():
#     user = User(id=1, name="Jan", age=15)
#     db.session.add(user)
#     db.session.commit()
#     yield user
#
#
#
# @pytest.fixture
# def logged_in_user(test_client):
#     user = User(id=100, name="George")
#     db.session.add(user)
#     with audit_logger.disable(db.session):
#         db.session.commit()
#
#     with test_client.application.test_request_context():
#         login_user(user)
#         yield user



from logging.config import fileConfig

from alembic import context
from sqlalchemy import engine_from_config, pool

from tests.defaults.flask_app import db

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
fileConfig(config.config_file_name)

target_metadata = db.metadata


def process_revision_directives(context, revision, directives):
    if getattr(config.cmd_opts, "autogenerate", False):
        script = directives[0]
        if script.upgrade_ops.is_empty():
            directives[:] = []
            print("No changes in schema detected.")


def run_migrations_online():
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """
    connectable = engine_from_config(
        config.get_section(config.config_ini_section),
        prefix="sqlalchemy.",
        poolclass=pool.NullPool,
    )

    with connectable.connect() as connection:
        context.configure(
            connection=connection,
            target_metadata=target_metadata,
            include_schemas=True,
            process_revision_directives=process_revision_directives,
        )

        with context.begin_transaction():
            context.run_migrations()


run_migrations_online()
